from typing import (
    Dict,
    Optional,
)


class OrderInsertPaymentResponse(object):
    def __init__(
        self,
        tender,                     # type: Dict
    ):
        # type: (...) -> None

        self.dueAmount = tender.get("dueAmount")
        self.tenderId = tender.get("tenderId")
