from application.model.configuration import Configurations


class TestConnectionInteractor(object):

    def __init__(self, configs):
        # type: (Configurations) -> None

        self.configs = configs
        self.logger = self.configs.logger

    def test_connection(self, current_pos_id):
        # type: (int) -> bool

        self.logger.info("Testing nCoupon connection. CurrentPosId: {}".format(current_pos_id))
        return True
