class Config(object):

    def __init__(self):
        self.loader_path = None
        self.component_name = None
        self.output_folder = None
        self.encode = None
        self.log_level = None
        self.connection_timeout = None
