class AuditEvent(object):
    PAID = "PAID"
    TABLE_CLOSED = "TABLE_CLOSED"
    VOID_ORDER = "VOID_ORDER"
    OPERATOR_LOGOUT = "OPERATORLOGOUT"
    PRODUCTION_ORDER_TIMES = "PRODUCTION_ORDER_TIMES"
    ORDER_REPROCESS = "ORDER_REPROCESS"
    CLOSE_STORE_DAY = "CLOSE_STORE_DAY"
    OPEN_STORE_DAY = "OPEN_STORE_DAY"
