from typing import List


class TableReportFilterDto(object):
    def __init__(
            self,
            pos_id,     # type: str
            table_id,   # type: str
            orders      # type: List[str]
    ):
        # type: (...) -> None

        self.pos_id = pos_id
        self.table_id = table_id
        self.orders = orders
