data = (
'ddwim',    # 0x00
'ddwib',    # 0x01
'ddwibs',    # 0x02
'ddwis',    # 0x03
'ddwiss',    # 0x04
'ddwing',    # 0x05
'ddwij',    # 0x06
'ddwic',    # 0x07
'ddwik',    # 0x08
'ddwit',    # 0x09
'ddwip',    # 0x0a
'ddwih',    # 0x0b
'ddyu',    # 0x0c
'ddyug',    # 0x0d
'ddyugg',    # 0x0e
'ddyugs',    # 0x0f
'ddyun',    # 0x10
'ddyunj',    # 0x11
'ddyunh',    # 0x12
'ddyud',    # 0x13
'ddyul',    # 0x14
'ddyulg',    # 0x15
'ddyulm',    # 0x16
'ddyulb',    # 0x17
'ddyuls',    # 0x18
'ddyult',    # 0x19
'ddyulp',    # 0x1a
'ddyulh',    # 0x1b
'ddyum',    # 0x1c
'ddyub',    # 0x1d
'ddyubs',    # 0x1e
'ddyus',    # 0x1f
'ddyuss',    # 0x20
'ddyung',    # 0x21
'ddyuj',    # 0x22
'ddyuc',    # 0x23
'ddyuk',    # 0x24
'ddyut',    # 0x25
'ddyup',    # 0x26
'ddyuh',    # 0x27
'ddeu',    # 0x28
'ddeug',    # 0x29
'ddeugg',    # 0x2a
'ddeugs',    # 0x2b
'ddeun',    # 0x2c
'ddeunj',    # 0x2d
'ddeunh',    # 0x2e
'ddeud',    # 0x2f
'ddeul',    # 0x30
'ddeulg',    # 0x31
'ddeulm',    # 0x32
'ddeulb',    # 0x33
'ddeuls',    # 0x34
'ddeult',    # 0x35
'ddeulp',    # 0x36
'ddeulh',    # 0x37
'ddeum',    # 0x38
'ddeub',    # 0x39
'ddeubs',    # 0x3a
'ddeus',    # 0x3b
'ddeuss',    # 0x3c
'ddeung',    # 0x3d
'ddeuj',    # 0x3e
'ddeuc',    # 0x3f
'ddeuk',    # 0x40
'ddeut',    # 0x41
'ddeup',    # 0x42
'ddeuh',    # 0x43
'ddyi',    # 0x44
'ddyig',    # 0x45
'ddyigg',    # 0x46
'ddyigs',    # 0x47
'ddyin',    # 0x48
'ddyinj',    # 0x49
'ddyinh',    # 0x4a
'ddyid',    # 0x4b
'ddyil',    # 0x4c
'ddyilg',    # 0x4d
'ddyilm',    # 0x4e
'ddyilb',    # 0x4f
'ddyils',    # 0x50
'ddyilt',    # 0x51
'ddyilp',    # 0x52
'ddyilh',    # 0x53
'ddyim',    # 0x54
'ddyib',    # 0x55
'ddyibs',    # 0x56
'ddyis',    # 0x57
'ddyiss',    # 0x58
'ddying',    # 0x59
'ddyij',    # 0x5a
'ddyic',    # 0x5b
'ddyik',    # 0x5c
'ddyit',    # 0x5d
'ddyip',    # 0x5e
'ddyih',    # 0x5f
'ddi',    # 0x60
'ddig',    # 0x61
'ddigg',    # 0x62
'ddigs',    # 0x63
'ddin',    # 0x64
'ddinj',    # 0x65
'ddinh',    # 0x66
'ddid',    # 0x67
'ddil',    # 0x68
'ddilg',    # 0x69
'ddilm',    # 0x6a
'ddilb',    # 0x6b
'ddils',    # 0x6c
'ddilt',    # 0x6d
'ddilp',    # 0x6e
'ddilh',    # 0x6f
'ddim',    # 0x70
'ddib',    # 0x71
'ddibs',    # 0x72
'ddis',    # 0x73
'ddiss',    # 0x74
'dding',    # 0x75
'ddij',    # 0x76
'ddic',    # 0x77
'ddik',    # 0x78
'ddit',    # 0x79
'ddip',    # 0x7a
'ddih',    # 0x7b
'ra',    # 0x7c
'rag',    # 0x7d
'ragg',    # 0x7e
'rags',    # 0x7f
'ran',    # 0x80
'ranj',    # 0x81
'ranh',    # 0x82
'rad',    # 0x83
'ral',    # 0x84
'ralg',    # 0x85
'ralm',    # 0x86
'ralb',    # 0x87
'rals',    # 0x88
'ralt',    # 0x89
'ralp',    # 0x8a
'ralh',    # 0x8b
'ram',    # 0x8c
'rab',    # 0x8d
'rabs',    # 0x8e
'ras',    # 0x8f
'rass',    # 0x90
'rang',    # 0x91
'raj',    # 0x92
'rac',    # 0x93
'rak',    # 0x94
'rat',    # 0x95
'rap',    # 0x96
'rah',    # 0x97
'rae',    # 0x98
'raeg',    # 0x99
'raegg',    # 0x9a
'raegs',    # 0x9b
'raen',    # 0x9c
'raenj',    # 0x9d
'raenh',    # 0x9e
'raed',    # 0x9f
'rael',    # 0xa0
'raelg',    # 0xa1
'raelm',    # 0xa2
'raelb',    # 0xa3
'raels',    # 0xa4
'raelt',    # 0xa5
'raelp',    # 0xa6
'raelh',    # 0xa7
'raem',    # 0xa8
'raeb',    # 0xa9
'raebs',    # 0xaa
'raes',    # 0xab
'raess',    # 0xac
'raeng',    # 0xad
'raej',    # 0xae
'raec',    # 0xaf
'raek',    # 0xb0
'raet',    # 0xb1
'raep',    # 0xb2
'raeh',    # 0xb3
'rya',    # 0xb4
'ryag',    # 0xb5
'ryagg',    # 0xb6
'ryags',    # 0xb7
'ryan',    # 0xb8
'ryanj',    # 0xb9
'ryanh',    # 0xba
'ryad',    # 0xbb
'ryal',    # 0xbc
'ryalg',    # 0xbd
'ryalm',    # 0xbe
'ryalb',    # 0xbf
'ryals',    # 0xc0
'ryalt',    # 0xc1
'ryalp',    # 0xc2
'ryalh',    # 0xc3
'ryam',    # 0xc4
'ryab',    # 0xc5
'ryabs',    # 0xc6
'ryas',    # 0xc7
'ryass',    # 0xc8
'ryang',    # 0xc9
'ryaj',    # 0xca
'ryac',    # 0xcb
'ryak',    # 0xcc
'ryat',    # 0xcd
'ryap',    # 0xce
'ryah',    # 0xcf
'ryae',    # 0xd0
'ryaeg',    # 0xd1
'ryaegg',    # 0xd2
'ryaegs',    # 0xd3
'ryaen',    # 0xd4
'ryaenj',    # 0xd5
'ryaenh',    # 0xd6
'ryaed',    # 0xd7
'ryael',    # 0xd8
'ryaelg',    # 0xd9
'ryaelm',    # 0xda
'ryaelb',    # 0xdb
'ryaels',    # 0xdc
'ryaelt',    # 0xdd
'ryaelp',    # 0xde
'ryaelh',    # 0xdf
'ryaem',    # 0xe0
'ryaeb',    # 0xe1
'ryaebs',    # 0xe2
'ryaes',    # 0xe3
'ryaess',    # 0xe4
'ryaeng',    # 0xe5
'ryaej',    # 0xe6
'ryaec',    # 0xe7
'ryaek',    # 0xe8
'ryaet',    # 0xe9
'ryaep',    # 0xea
'ryaeh',    # 0xeb
'reo',    # 0xec
'reog',    # 0xed
'reogg',    # 0xee
'reogs',    # 0xef
'reon',    # 0xf0
'reonj',    # 0xf1
'reonh',    # 0xf2
'reod',    # 0xf3
'reol',    # 0xf4
'reolg',    # 0xf5
'reolm',    # 0xf6
'reolb',    # 0xf7
'reols',    # 0xf8
'reolt',    # 0xf9
'reolp',    # 0xfa
'reolh',    # 0xfb
'reom',    # 0xfc
'reob',    # 0xfd
'reobs',    # 0xfe
'reos',    # 0xff
)
