from typing import Dict


class Action(object):
    def __init__(
        self,
        obj,     # type: Dict
    ):
        # type: (...) -> None

        self.order = obj.get("order", None)
        self.name = obj.get("name")
        self.value = obj.get("value")
        self.times_to_apply = None
