from sysactions import action, get_model, print_report


@action
def show_pos_versions(pos_id, pos_versions_json):
    model = get_model(pos_id)
    print_report(pos_id, model, True, "versions_report", pos_id, pos_versions_json)
